
rm(list = ls())

#####################################################
##### Compare DGL and tMVN in real applications #####
#####################################################

library(tmg)
library(FastGP)
library(bsamGP)
require(Rcpp)
#sourceCpp("rcpptry2.cpp")
sourceCpp("rcpptry_cpp.cpp") # alternative to "rcpptry2.cpp"

source("all_base_functions.R")
source("all_models.R")

#### Load real data sets #####

########################################
######## Age and income data ###########
########################################
#install.packages("SemiPar")
require(SemiPar)
data(age.income)
attach(age.income)

v1=age; y1=log.income; v=sort(v1); y=y1[order(v1)]
plot(v,y,type="p",pch=20,xlab="age",ylab="log.income")

# determine the flat region 
reg_f = function(x) (x>=26)

####### Running constrained models with DGL and tMVN priors #######

##### DGL with Matern kernel #####
post.sam=mon.shrink.hyp(y,v/max(v),N=round(length(v)/8),delta_tau=0.5,
                        sseed=12345,mcmc=10000,brn=5000,thin=10,
                        eta=50,return.plot=FALSE,return.traplot=FALSE)

ub=max(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
lb=min(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
(waic=fun.waic(y,post.sam$fhat_sam,post.sam$sig_sam))  # total WAIC 
(waic.flt=fun.waic.reg(v,y,post.sam$fhat_sam,post.sam$sig_sam,reg_f))# WAIC for the flat region only 

plot(sort(v),y[order(v)],pch=20,main="DGL prior",ylim=c(lb,ub),
     cex.main=1.3,col="green3",xlab="age",ylab="log.income",
     sub=paste("WAIC =",round(waic,3)),cex.sub=1.2)
lines(sort(v),post.sam$fmean[order(v)],lwd=2)
lines(sort(v),post.sam$f_low[order(v)],lwd=2,lty=2,col="blue")
lines(sort(v),post.sam$f_upp[order(v)],lwd=2,lty=2,col="blue")


##### tMVN with an improper prior on \tau^2 #####
post.sam=mon.ESS.hyp(y,v/max(v),N=round(length(v)/8),
                     sseed=12345,mcmc=10000,brn=5000,thin=10,
                     eta=50,return.plot=FALSE,return.traplot=FALSE)

ub=max(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
lb=min(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
(waic=fun.waic(y,post.sam$fhat_sam,post.sam$sig_sam))  # total WAIC 
(waic.flt=fun.waic.reg(v,y,post.sam$fhat_sam,post.sam$sig_sam,reg_f)) # WAIC for the flat region only

plot(sort(v),y[order(v)],pch=20,main="tMVN prior",ylim=c(lb,ub),
     cex.main=1.3,col="green3",xlab="age",ylab="log.income",
     sub=paste("WAIC =",round(waic,3)),cex.sub=1.2)
lines(sort(v),post.sam$fmean[order(v)],lwd=2)
lines(sort(v),post.sam$f_low[order(v)],lwd=2,lty=2,col="blue")
lines(sort(v),post.sam$f_upp[order(v)],lwd=2,lty=2,col="blue") 

detach(age.income)


#####################################################
######## Light detection and ranging data ###########
#####################################################
# install.packages("HRW")
require(HRW)
data(lidar)
attach(lidar)

v1=range; y1=logratio; v=sort(v1); y=y1[order(v1)]
plot(v,y,type="p",pch=20,xlab="range",ylab="logratio")

# determine the flat region 
reg_f = function(x) (x<=550)

####### Running constrained models with DGL and tMVN priors #######

##### DGL with Matern kernel (non-increasing) #####
post.sam=dec.shrink.hyp(y,v/max(v),N=round(length(v)/8),eta=50,delta_tau=0.5,
                        sseed=12345,mcmc=10000,brn=5000,thin=10,
                        return.plot=FALSE,return.traplot=FALSE)

ub=max(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
lb=min(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
(waic=fun.waic(y,post.sam$fhat_sam,post.sam$sig_sam)) # total WIAC
(waic.flt=fun.waic.reg(v,y,post.sam$fhat_sam,post.sam$sig_sam,reg_f))  # WAIC for the flat region only

plot(sort(v),y[order(v)],pch=20,main="DGL prior",ylim=c(lb,ub),
     cex.main=1.3,col="green3",xlab="range",ylab="logratio",
     sub=paste("WAIC =",round(waic,3)),cex.sub=1.2)
lines(sort(v),post.sam$fmean[order(v)],lwd=2)
lines(sort(v),post.sam$f_low[order(v)],lwd=2,lty=2,col="blue")
lines(sort(v),post.sam$f_upp[order(v)],lwd=2,lty=2,col="blue")


##### tMVN with an improper prior on \tau^2 (non-increasing) #####
post.sam=mon.dec.ESS.hyp(y,v/max(v),N=round(length(v)/8),
                         sseed=12345,mcmc=10000,brn=5000,thin=10,
                         eta=50,return.plot=FALSE,return.traplot=FALSE)

ub=max(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
lb=min(post.sam$f_low,post.sam$f_upp,post.sam$fmean,y)
(waic=fun.waic(y,post.sam$fhat_sam,post.sam$sig_sam)) # total WIAC
(waic.flt=fun.waic.reg(v,y,post.sam$fhat_sam,post.sam$sig_sam,reg_f))  # WAIC for the flat region only


plot(sort(v),y[order(v)],pch=20,main="tMVN prior",ylim=c(lb,ub),
     cex.main=1.3,col="green3",xlab="range",ylab="logratio", 
     sub=paste("WAIC =",round(waic,3)),cex.sub=1.2)
lines(sort(v),post.sam$fmean[order(v)],lwd=2)
lines(sort(v),post.sam$f_low[order(v)],lwd=2,lty=2,col="blue")
lines(sort(v),post.sam$f_upp[order(v)],lwd=2,lty=2,col="blue") 

detach(lidar)



###### END #####